# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.

#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import codecs
import os
import os.path

import fixtures
from oslo_config import fixture as config
from oslotest import base as test_base

from oslo_policy import _checks
from oslo_policy import policy


class PolicyBaseTestCase(test_base.BaseTestCase):

    def setUp(self):
        super(PolicyBaseTestCase, self).setUp()
        self.conf = self.useFixture(config.Config()).conf
        self.config_dir = self.useFixture(fixtures.TempDir()).path
        self.conf(args=['--config-dir', self.config_dir])
        self.enforcer = policy.Enforcer(self.conf)
        self.addCleanup(self.enforcer.clear)

    def get_config_file_fullname(self, filename):
        return os.path.join(self.config_dir, filename.lstrip(os.sep))

    def create_config_file(self, filename, contents):
        """Create a configuration file under the config dir.

        Also creates any intermediate paths needed so the file can be
        in a subdirectory.

        """
        path = self.get_config_file_fullname(filename)
        pardir = os.path.dirname(path)
        if not os.path.exists(pardir):
            os.makedirs(pardir)
        with codecs.open(path, 'w', encoding='utf-8') as f:
            f.write(contents)


class FakeCheck(_checks.BaseCheck):
    def __init__(self, result=None):
        self.result = result

    def __str__(self):
        return str(self.result)

    def __call__(self, target, creds, enforcer):
        if self.result is not None:
            return self.result
        return (target, creds, enforcer)



#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import sys

from oslo_serialization import jsonutils

from oslo_policy import policy


def _try_rule(key, rule, target, access_data, o):
    try:
        result = rule(target, access_data, o)
        if result:
            print("passed: %s" % key)
        else:
            print("failed: %s" % key)
    except Exception as e:
        print(e)
        print("exception: %s" % rule)


def tool(policy_file, access_file, apply_rule, is_admin=False):
    access = access_file.read()
    access_data = jsonutils.loads(access)['token']
    access_data['roles'] = [role['name'] for role in access_data['roles']]
    access_data['project_id'] = access_data['project']['id']
    access_data['is_admin'] = is_admin
    policy_data = policy_file.read()
    rules = policy.Rules.load(policy_data, "default")

    class Object(object):
        pass
    o = Object()
    o.rules = rules

    target = {"project_id": access_data['project_id']}

    if apply_rule:
        key = apply_rule
        rule = rules[apply_rule]
        _try_rule(key, rule, target, access_data, o)
        return
    for key, rule in rules.items():
        if ":" in key:
            _try_rule(key, rule, target, access_data, o)


def main():
    parser = argparse.ArgumentParser(sys.argv[0])
    parser.add_argument(
        '--policy',
        required=True,
        type=argparse.FileType('rb', 0),
        help='path to a policy file')
    parser.add_argument(
        '--access',
        required=True,
        type=argparse.FileType('rb', 0),
        help='path to a file containing OpenStack Identity API' +
        ' access info in JSON format')
    parser.add_argument(
        '--rule',
        help='rule to test')

    parser.add_argument(
        '--is_admin',
        help='set is_admin=True on the credentials used for the evaluation')

    args = parser.parse_args()
    try:
        is_admin = args.is_admin.lower() == "true"
    except Exception:
        is_admin = False
    tool(args.policy, args.access, args.rule, is_admin)


if __name__ == "__main__":
    sys.exit(main())



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__all__ = ['HttpCheckFixture', 'HttpsCheckFixture']

import fixtures


class HttpCheckFixture(fixtures.Fixture):
    """Helps short circuit the external http call"""

    def __init__(self, return_value=True):
        """Initialize the fixture.

        :param return_value: True implies the policy check passed and False
               implies that the policy check failed
        :type return_value: boolean
        """
        super(HttpCheckFixture, self).__init__()
        self.return_value = return_value

    def setUp(self):
        super(HttpCheckFixture, self).setUp()

        def mocked_call(target, cred, enforcer, rule):
            return self.return_value

        self.useFixture(
            fixtures.MonkeyPatch(
                'oslo_policy._external.HttpCheck.__call__',
                mocked_call,
            )
        )


class HttpsCheckFixture(fixtures.Fixture):
    """Helps short circuit the external http call"""

    def __init__(self, return_value=True):
        """Initialize the fixture.

        :param return_value: True implies the policy check passed and False
               implies that the policy check failed
        :type return_value: boolean
        """
        super(HttpsCheckFixture, self).__init__()
        self.return_value = return_value

    def setUp(self):
        super(HttpsCheckFixture, self).setUp()

        def mocked_call(target, cred, enforcer, rule):
            return self.return_value

        self.useFixture(
            fixtures.MonkeyPatch(
                'oslo_policy._external.HttpsCheck.__call__',
                mocked_call,
            )
        )



# Copyright 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import textwrap

from oslotest import base

from oslo_policy import policy
from oslo_policy import sphinxext


class FormatPolicyTest(base.BaseTestCase):

    def test_minimal(self):
        results = '\n'.join(list(sphinxext._format_policy_section(
            'foo', [policy.RuleDefault('rule_a', '@')])))

        self.assertEqual(textwrap.dedent("""
        foo
        ===

        ``rule_a``
            :Default: ``@``

            (no description provided)
        """).lstrip(), results)

    def test_with_description(self):
        results = '\n'.join(list(sphinxext._format_policy_section(
            'foo', [policy.RuleDefault('rule_a', '@', 'My sample rule')]
        )))

        self.assertEqual(textwrap.dedent("""
        foo
        ===

        ``rule_a``
            :Default: ``@``

            My sample rule
        """).lstrip(), results)

    def test_with_operations(self):
        results = '\n'.join(list(sphinxext._format_policy_section(
            'foo', [policy.DocumentedRuleDefault(
                'rule_a', '@', 'My sample rule', [
                    {'method': 'GET', 'path': '/foo'},
                    {'method': 'POST', 'path': '/some'}])]
        )))

        self.assertEqual(textwrap.dedent("""
        foo
        ===

        ``rule_a``
            :Default: ``@``
            :Operations:
                - **GET** ``/foo``
                - **POST** ``/some``

            My sample rule
        """).lstrip(), results)

    def test_with_scope_types(self):
        operations = [
            {'method': 'GET', 'path': '/foo'},
            {'method': 'POST', 'path': '/some'}
        ]
        scope_types = ['bar']
        rule = policy.DocumentedRuleDefault(
            'rule_a', '@', 'My sample rule', operations,
            scope_types=scope_types
        )

        results = '\n'.join(list(sphinxext._format_policy_section(
            'foo', [rule]
        )))

        self.assertEqual(textwrap.dedent("""
        foo
        ===

        ``rule_a``
            :Default: ``@``
            :Operations:
                - **GET** ``/foo``
                - **POST** ``/some``
            :Scope Types:
                - **bar**

            My sample rule
        """).lstrip(), results)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib
import copy
import os

from oslo_policy import _checks
from oslo_policy._i18n import _
from oslo_serialization import jsonutils
import requests


class HttpCheck(_checks.Check):
    """Check ``http:`` rules by calling to a remote server.

    This example implementation simply verifies that the response
    is exactly ``True``.
    """

    def __call__(self, target, creds, enforcer, current_rule=None):
        url = ('http:' + self.match) % target
        data, json = self._construct_payload(creds, current_rule,
                                             enforcer, target)
        with contextlib.closing(
                requests.post(url, json=json, data=data)
        ) as r:
            return r.text.lstrip('"').rstrip('"') == 'True'

    @staticmethod
    def _construct_payload(creds, current_rule, enforcer, target):
        # Convert instances of object() in target temporarily to
        # empty dict to avoid circular reference detection
        # errors in jsonutils.dumps().
        temp_target = copy.deepcopy(target)
        for key in target.keys():
            element = target.get(key)
            if type(element) is object:
                temp_target[key] = {}
        data = json = None
        if (enforcer.conf.oslo_policy.remote_content_type ==
                'application/x-www-form-urlencoded'):
            data = {'rule': jsonutils.dumps(current_rule),
                    'target': jsonutils.dumps(temp_target),
                    'credentials': jsonutils.dumps(creds)}
        else:
            json = {'rule': current_rule,
                    'target': temp_target,
                    'credentials': creds}
        return data, json


class HttpsCheck(HttpCheck):
    """Check ``https:`` rules by calling to a remote server.

    This example implementation simply verifies that the response
    is exactly ``True``.
    """

    def __call__(self, target, creds, enforcer, current_rule=None):
        url = ('https:' + self.match) % target

        cert_file = enforcer.conf.oslo_policy.remote_ssl_client_crt_file
        key_file = enforcer.conf.oslo_policy.remote_ssl_client_key_file
        ca_crt_file = enforcer.conf.oslo_policy.remote_ssl_ca_crt_file
        verify_server = enforcer.conf.oslo_policy.remote_ssl_verify_server_crt

        if cert_file:
            if not os.path.exists(cert_file):
                raise RuntimeError(
                    _("Unable to find ssl cert_file  : %s") % cert_file)
            if not os.access(cert_file, os.R_OK):
                raise RuntimeError(
                    _("Unable to access ssl cert_file  : %s") % cert_file)
        if key_file:
            if not os.path.exists(key_file):
                raise RuntimeError(
                    _("Unable to find ssl key_file : %s") % key_file)
            if not os.access(key_file, os.R_OK):
                raise RuntimeError(
                    _("Unable to access ssl key_file  : %s") % key_file)
        cert = (cert_file, key_file)
        if verify_server:
            if ca_crt_file:
                if not os.path.exists(ca_crt_file):
                    raise RuntimeError(
                        _("Unable to find ca cert_file  : %s") % ca_crt_file)
                verify_server = ca_crt_file

        data, json = self._construct_payload(creds, current_rule,
                                             enforcer, target)
        with contextlib.closing(
                requests.post(url, json=json,
                              data=data, cert=cert,
                              verify=verify_server)
        ) as r:
            return r.text.lstrip('"').rstrip('"') == 'True'



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import fixture as config
from oslotest import base as test_base

from oslo_policy import opts


class OptsTestCase(test_base.BaseTestCase):

    def setUp(self):
        super(OptsTestCase, self).setUp()
        self.conf = self.useFixture(config.Config()).conf

    def test_set_defaults_policy_file(self):
        opts._register(self.conf)
        self.assertNotEqual('new-value.json',
                            self.conf.oslo_policy.policy_file)
        opts.set_defaults(self.conf, policy_file='new-value.json')
        self.assertEqual('new-value.json',
                         self.conf.oslo_policy.policy_file)
